;****************************************************************************
;
;                           KISS MODE, TOP LEVEL MODULE
;
; BY REMI BILODEAU, NOVEMBER 16 1994
;
; This kiss mode module are implemented to be compatible with KAM KPC-3 TNC
;
;****************************************************************************
 
;****************************************************************************
;
;                           EQUATE FOR KISS MODE
;
;****************************************************************************
FEND            EQU     0C0h    ; End of stream caracter
FESC            EQU     0DBh    ; Special escape caracter
TFEND           EQU     0DCh    ; To insert FEND char in data, send FESC+TFEND
TFESC           EQU     0DDh    ; To insert FESC char in data, send FESC+TFESC       

KISSTXptr       EQU     01Eh    ; Pointer to TX buffer for KISS mode
KissStatus      EQU     02Dh    ; Status of KISS mode
KISSTXsize      EQU     02Eh    ; Current size of TX buffer

NULL            EQU     0       ; NULL caracter
 
;****************************************************************************
; 
;                              MAIN OF KISS MODULE 
;
; Documentation:
; 1- Check for SLOTTIME/PERSISTANCE to transmit packet with TXDELAY
; 2- Watch receiver, and send in KISS format packet received with good CRC
; 3- Watch serial port and take this action:
;    a) Send KISS format byte stream from serial port to packet queue if need
;    b) Set TXDELAY if need
;    c) Set PERSIST if need
;    d) Set SLOTTIME if neeed
;    e) go out of KISS mode if need
; 4- loop to 1
;
; Status byte definition:
; Bit 0-3 = Command ID:  0000 No command are define
;                        0001 Next caracter is byte for data transfer
;                        0010 Next caracter is parameter for TXDELAY
;                        0011 Next caracter is parameter for PERSISTENCE
;                        0100 Next caracter is parameter for SLOTTIME
;                        0101 End KISS mode on next FEND char
; Bit 4   = FEND caracter are detected (nexts chars is on byte stream)
; Bit 5   = Next caracter is a escaped sequence (last is FESC)
;
; Link time-out has been estabished, after 5 seconds of inactivity of serial
; port the Start FEND flag is cleared. If communication is broken beetween
; host and KISS TNC, the link is resetted automaticly. (use of T2 timer)
;
; STA led is used to indicate packet in TXQ buffer
; CON led is used to indicate KISS serial transmission 
;****************************************************************************
KISSmain:
         
        CLR     STAled          ; No packet in TXQ
        CLR     CONled          ; No KISS transmission
        CLR     T2run           ; T2 inactive
        MOV     T2count,#50     ; 5 sec time-out
        MOV     KissStatus,#0   ; Reset status
        MOV     dptr,#768
        CALL    MemALLOC        ; Allocate 768 bytes for TX kiss buffer
        MOVX    @dptr,a         ; Store handle on first byte
        MOV     KissTXptr,dpl
        MOV     KissTXptr+1,dph
        MOV     KissTXsize,#0   ; No byte in queue
        MOV     KissTXsize+1,#0

Kiss0:  CALL    CheckPersist    ; Make slottime/persist process
        CALL    CheckIncomingPkt ; Watch incoming packet and transmit to serial
         
        CALL    SerialGET
        JNC     Kisa            ; No caracter availaible, bypass process
        JMP     Kiss3

Kisa:   MOV     b,a             ; Save caracter
        MOV     a,KissStatus
        JB      Abit4,Kiss2a
         
        MOV     a,b
        XRL     a,#FEND         ; Start for starting FEND
        JZ      Kisb            ; If finded, reset all variable
        JMP     Kiss3

Kisb:   MOV     KissStatus,#10h ; Reset KISS status with FEND finded
        MOV     KissTXsize,#0   ; No byte in queue
        MOV     KissTXsize+1,#0
        SETB    T2run           ; Start time-out timer
        SETB    CONled          ; KISS transmission begin
        JMP     Kiss3

Kiss2a: MOV     T2count,#50     ; Reset timer value, caracter received
         
        MOV     a,KissStatus        ; Check if command have been received
        ANL     a,#00001111b
        JNZ     Kiss2b 
         
        MOV     a,b             ; Decode KISS command byte          
        
        INC     a               ; Detect code 255 to exit KISS mode
        JNZ     Kiss2c
        MOV     KissStatus,#15h ; Exit code finded, exit KISS on final FEND
        JMP     Kiss3

Kiss2c: ANL     a,#00001111b    ; Kiss code, 1-15
        ORL     a,#00010000b    ; FEND detected (same status)
        MOV     KissStatus,a
        JMP     Kiss3


Kiss2b:  
        MOV     a,b
        XRL     a,#FEND
        JZ      Kis2d
        JMP     Kiss2d
Kis2d:   
        CLR     T2run                   ; Stop time-out time
        CLR     CONled
        MOV     T2count,#50             ; Reset it value

        MOV     a,KissStatus
        ANL     a,#00001111b
        XRL     a,#5
        JNZ     Kiss2e
                 MOV     dpl,KissTXptr           ; Case of exit code
        MOV     dph,KissTXptr+1
        MOVX    a,@dptr
        CALL    MemFREE
        RET                             ; In case of code 255 (exit)


Kiss2e: MOV     a,KissStatus
        ANL     a,#00001111b
        DEC     a
        JZ      other1        
        JMP     other
other1:  
        MOV     a,KissTXsize            ; Case of data transfer
        JNZ     kis0                    ; Watch if size=0
        MOV     a,KissTXsize+1
        JNZ     kis0
        JMP     other

Kis0:   MOV     a,TXQhandle             ; Scan for pointer for data block
        CALL    MemGETPOINTER

Kis1:    
        MOVX    a,@dptr                 ; Save size in R5:R4
        MOV     R4,a
        INC     dptr
        MOVX    a,@dptr
        MOV     R5,a
        INC     dptr

        MOV     a,R4                    ; Detect size NULL
        ADD     a,R5
        JZ      Kis2

        MOV     a,dpl                   ; update DPTR
        ADD     a,R4
        MOV     dpl,a
        MOV     a,dph
        ADDC    a,R5
        MOV     dph,a
        JMP     Kis1

Kis2:   MOV     a,dpl                   ; DPTR-=2
        CLR     C
        SUBB    a,#2
        MOV     dpl,a
        MOV     a,dph
        SUBB    a,#0
        MOV     dph,a

        MOV     R6,dpl
        MOV     R7,dph                  ; Save DPTR

        

         
        MOV     a,dpl                   ; Add current TXQ pointer to packet size
        ADD     a,KissTXsize
        MOV     R4,a
        MOV     a,dph
        ADDC    a,KissTXsize+1
        MOV     R5,a
        MOV     a,R4               
        ADD     a,#2
        MOV     R4,a
        MOV     a,R5
        ADDC    a,#0
        MOV     R5,a

        CLR     C                       ; Sub result from start pointer
        MOV     a,R4
        SUBB    a,R6
        MOV     R4,a
        MOV     a,R5
        SUBB    a,R7
        MOV     R5,a

        CLR     C                       ; Sub result from TXQ total size
        MOV     a,R4
        SUBB    a,#low(TXQsize)
        MOV     R4,a
        MOV     a,R5
        SUBB    a,#high(TXQsize)
        MOV     R5,a                    ; Carry indicate the overflow


        JC      Kis3
                 
Kis2a:  CALL    CheckPersist    ; check persist/slottime parameter
        CALL    CheckIncomingPkt ; Watch incoming packet and transmit to serial
        JB      TXQempty,Kis2a
        MOV     a,TXQhandle     ; Scan for pointer for data block
        CALL    MemGETPOINTER
        MOV     R6,dpl
        MOV     R7,dph                  ; Resave DPTR (in case of modifying)           
     

Kis3:    
        MOV     a,KissTXsize   
        ADD     a,#2                    ; size for CRC         
        MOVX    @dptr,a
        INC     dptr
        MOV     a,KissTXsize+1
        ADDC    a,#0
        MOVX    @dptr,a
        INC     dptr
        
        SETB    RS0                     ; Transfer block
        MOV     R2,dpl
        MOV     R3,dph
        MOV     dpl,KissTXptr
        MOV     dph,KissTXptr+1
        INC     dptr
        MOV     R0,dpl
        MOV     R1,dph
        MOV     R4,KissTXsize
        MOV     R5,KissTXsize+1
        CALL    MemCOPY
        CLR     RS0

        MOV     dpl,R6                  ; Calculate CRC
        MOV     dph,R7
        CALL    CRCCheck

        CLR     a                       ; Size is NULL to indicate the end
        MOVX    @dptr,a
        INC     dptr
        MOVX    @dptr,a

        SETB    TXQempty                ; Stuff in TX queue
        SETB    STAled                  ; TXQ buffer are not empty

        
        
        MOV     KissTXsize,#0
        MOV     KissTXsize+1,#0




Other:   
        MOV     KissStatus,#0           ; Case of TX Parameter command
        JMP     Kiss3             
      
          
Kiss2d:  
         
        MOV     a,KissStatus
        ANL     a,#00001111b        
        XRL     a,#2
        JNZ     Kiss2f
        MOV     a,b
        MOV     TXDELAY,a
        JMP     Kiss3

         
Kiss2f: MOV     a,KissStatus
        ANL     a,#00001111b        
        XRL     a,#3
        JNZ     Kiss2g
        MOV     a,b
        MOV     Persist,a
        JMP     Kiss3

         
Kiss2g: MOV     a,KissStatus
        ANL     a,#00001111b        
        XRL     a,#4
        JNZ     Kiss2h
        MOV     a,b
        MOV     SlotTime,a
        JMP     Kiss3

         
Kiss2h: MOV     a,KissStatus
        ANL     a,#00001111b        
        XRL     a,#1
        JNZ     Kiss3

         
        MOV     a,b                     ; Check for escape sequence
        XRL     a,#FESC
        JNZ     Kiss2i
        MOV     a,KissStatus
        SETB    Abit5
        MOV     KissStatus,a
        JMP     Kiss3


Kiss2i: MOV     a,KissTXptr             ; Pointer to next written byte
        ADD     a,KissTXsize
        MOV     dpl,a
        MOV     a,KissTXptr+1
        ADDC    a,KissTXsize+1
        MOV     dph,a
        INC     dptr
        
        MOV     a,KissStatus
        JNB     Abit5,Kiss2j
         
        MOV     a,b
        XRL     a,#TFESC
        JNZ     Kiss2m
        MOV     a,#FESC                 ; Store FESC caracter
        MOVX    @dptr,a
        MOV     a,KissStatus
        CLR     Abit5
        MOV     KissStatus,a
        JMP     Kiss2k

Kiss2m: MOV     a,b
        XRL     a,#TFEND
        JNZ     Kiss3
        MOV     a,#FEND                 ; Store FEND caracter
        MOVX    @dptr,a
        MOV     a,KissStatus
        CLR     Abit5
        MOV     KissStatus,a
        JMP     Kiss2k

Kiss2j: MOV     a,b
        MOVX    @dptr,a                 ; Store caracter

Kiss2k: INC     KissTXsize              ; Update pointer
        MOV     a,KissTXsize
        JNZ     Kiss2l
        INC     KissTXsize+1
Kiss2l: JMP     Kiss3







Kiss3:   
        JNB     T2run,Kiss4
        MOV     a,T2count
        JNZ     Kiss4
        MOV     KissStatus,#0   ; time-out detected, reset link
        CLR     T2run
        CLR     CONled
        MOV     T2count,#50     ; 5 sec time-out


Kiss4:  JMP     Kiss0

 
;****************************************************************************
;
;           SEND RECEIVED PACKET IN KISS MODE FORMAT TO SERIAL PORT
;
; Documentation:
; 1-Scan in RX queue and send all good packet
; 2-flush all RX queue
;
;
; R5:R4 = Size of packet
; R7:R6 = DPTR save
;****************************************************************************
SendKISS:
        MOV     a,RXQhandle
        CALL    MemGETPOINTER
        MOV     R6,dpl                  ; Save DPTR in R7:R6
        MOV     R7,dph

SKISS0:  
        MOV     dpl,R6                  ; Set DPTR(bug with bad CRC, TNC lockup)
        MOV     dph,R7
        MOVX    a,@dptr                 ; Get size of packet
        MOV     R4,a  
        INC     dptr
        MOVX    a,@dptr
        MOV     R5,a          

         
        MOV     a,R4
        JNZ     SKISS1
        MOV     a,R5
        JNZ     SKISS1
        JMP     SKISS9

SKISS1:  
        MOV     dpl,R6
        MOV     dph,R7
        CALL    CRCCheck        ; Check CRC of packet
        MOV     a,dpl   
        MOV     dpl,R6
        MOV     R6,a
        MOV     a,dph
        MOV     dph,R7
        MOV     R7,a            ; Exchange R7:R6 <-> DPTR
        JC      SKISS0          ; In case of bad packet, do not send
        INC     dptr
        INC     dptr            ; R7:R6 on next pkt, dptr on data of good pkt

         
        MOV     a,R4            ; Erase CRC at end of packet
        CLR     C
        SUBB    a,#2
        MOV     R4,a
        JNC     Oksize
        DEC     R5

Oksize: MOV     a,#FEND         ; Start a new byte stream
        CALL    SerialTX
        MOV     a,#NULL         ; Send command: data stream
        CALL    SerialTX
        
NewByt: MOVX    a,@dptr         ; Get caracter to send
        INC     dptr
        MOV     b,a
        XRL     a,#FEND         ; If is FEND caracter?
        JZ      IsFEND
        MOV     a,b
        XRL     a,#FESC         ; If is FESC caracter?
        JZ      IsFESC
        MOV     a,b
        CALL    SerialTX        ; Send unprocessed caracter

Update: DEC     R4              ; Decrement R5:R4
        CJNE    R4,#255,Up0
        DEC     R5
Up0:    MOV     a,R4
        JNZ     NewByt
        MOV     a,R5
        JNZ     NewByt

        MOV     a,#FEND         ; stop the byte stream
        CALL    SerialTX
SKISS9: RET

 
;----------------------- FEND finded in received packet
IsFEND: MOV     a,#FESC
        CALL    SerialTX
        MOV     a,#TFEND
        CALL    SerialTX
        JMP     Update        

 
;----------------------- FESC finded in received packet
IsFESC: MOV     a,#FESC
        CALL    SerialTX
        MOV     a,#TFESC
        CALL    SerialTX
        JMP     Update        



 
;****************************************************************************
;
;           CHECK PERSISTENCE/SLOTTIME TO TRANSMIT ENQUEUTED PACKET
;
;****************************************************************************
CheckPersist:
        MOV     a,SlotCount     ; If SlotCount=0 -> TX queue have packet and
        JNZ     CP0             ; it the time to proceed persistance

        CALL    RANDOMIZE       ; Process Persistance
        CLR     C
        SUBB    a,Persist
        JNC     CP0
        CLR     STAled          ; clear TXQ buffer
        CALL    PacketEMPTYQUEUE ; Flush all TX queue
CP0:    RET
 
;****************************************************************************
;
;      WATCH FOR INCOMING PACKET AND, IF NEED, TRANSMIT TO SERIAL PORT
;
;****************************************************************************
CheckIncomingPkt:
        JNB     DCD,CI0
        CALL    PacketRECEIVE   ; Receive packet
        SETB    CONled          ; KISS transmission start
        CALL    SendKISS        ; Send in kiss format the good packet received
        CLR     CONled          ; KISS transmission terminate
CI0:    RET



