;****************************************************************************
;
;                      DYNAMIC MEMORY MANAGER VERSION 1.0
;
; BY REMI BILODEAU OCTOBER 13, 1994
;
; 8051 VERSION
;
;****************************************************************************
 
;****************************************************************************
; 
;                       DOCUMENTATION ON MEMORY MANAGER
;
; Description of variable:
; MEMENTRY   -    16 bit pointer in internal memory to block entry table
;                 Scratch pad address: FEh-FFh
;
; Block Entry Table(at begin of external RAM area):
; BYTE       -    Number of handle currently definited (0 to maximum entry)
; WORD       -    Currently remaining RAM (Free RAM)
; BYTE       -    Block handle #1 
; WORD       -    Block pointer #1
; BYTE       -    Block handle #2
; WORD       -    Block pointer #2
; ...
; WORD       -    Pointer to next free RAM block
;
; Function description:
; MEMINIT    -    Detect RAM chip location and size and initialize driver 
; MEMALLOC   -    Allocate block of memory, size of block in byte in DPTR.
;                 If succesfull handle are returned in A and pointer to 
;                 block in DPTR, else MEMERROR are called.
; MEMFREE    -    Free previous allocated block. Give handle in A, if function
;                 fail MEMERROR are called else normal return.
; MEMGETFREE -    Return in DPTR the remaining free memory.
; MEMGETPOINTER - Give in A the handle of block, pointer to it returned in DPTR
; MEMGETFREEPTR - Return pointer to free memory.
; 
;****************************************************************************


 
;****************************************************************************
;
;                            EQUIVALENCE FOR DMM
;
;****************************************************************************

MemEntry        equ     30h             ; Internal pointer to entry table
MaxEntry        equ     64              ; Maximum entry allowed in table

StartScan       equ     0E000h          ; Start Ram scanning at E000h
StopScan        equ     0FFFFh          ; Stop scanning here
 
Dec_DPTR        MACRO
        PUSH    ACC
        CLR     C
        MOV     a,dpl
        SUBB    a,#1
        MOV     dpl,a
        MOV     a,dph
        SUBB    a,#0
        MOV     dph,a
        POP     ACC
ENDM



 
;****************************************************************************
;
;                       MEMINIT - INITIALIZE MEMORY MANAGER
;
; -Scan position and size of RAM chip, setup MEMENTRY internal variable
; -Create empty block entry table
;
;****************************************************************************
MemINIT:
        mov     dptr,#StartScan                 ; Start scanning here
        
         
; R1:R0 returned at begin of scannable area, DPTR at begin of writtable
; area
;
Scan0:  clr     a               ; Write 00000000 into RAM
        movx    @dptr,a
        movx    a,@dptr
        jnz     Scan1
        dec     a               ; Write 11111111 into RAM
        movx    @dptr,a
        movx    a,@dptr  
        inc     a
        jnz     Scan1
        mov     R1,dph          ; bottom of RAM chip finded
        mov     R0,dpl
        sjmp    Scan2

Scan1:  inc     dptr            ; Cannot write here, continue scan
        mov     a,dpl
        cjne    a,#LOW(StopScan+1),Scan0
        mov     a,dph
        cjne    a,#HIGH(StopScan+1),Scan0
        mov     a,#0
        ljmp    MemError

         
; Scanning for top of RAM chip 
; 
;

Scan2:  clr     a               ; Write 00000000 on RAM
        movx    @dptr,a
        movx    a,@dptr
        jnz     Scan3
        dec     a               ; Write 11111111 on RAM
        movx    @dptr,a
        movx    a,@dptr  
        inc     a
        jnz     Scan3

        inc     dptr            ; continue scan if memory are there
        mov     a,dpl
        cjne    a,#LOW(StopScan+1),Scan2
        mov     a,dph
        cjne    a,#HIGH(StopScan+1),Scan2

Scan3:  mov     R3,dph          ; Memory scan are finish
        mov     R2,dpl          ; Set R3:R2 to last writtable byte plus one

                  
        mov     dpl,R0          ; Write byte at begin of ram chip
        mov     dph,R1
        mov     a,#55h
        movx    @dptr,a
        inc     dptr

Scan4:  movx    a,@dptr         ; Search image of byte writted
        xrl     a,#55h
        jz      Scan6
        inc     dptr
        mov     a,R3
        xrl     a,dph
        jnz     Scan4
        mov     a,R2
        xrl     a,dpl
        jnz     Scan4

Scan6:  mov     R2,dpl          ; Re-Setup R3:R2 with result of image scan
        mov     R3,dph

         
        mov     dpl,R0          ; Setup Entry Table location
        mov     dph,R1
        mov     a,R0
        mov     R0,#MemEntry
        mov     @R0,a                  
        inc     R0
        mov     a,R1
        mov     @R0,a

        clr     a               ; Setup entry table
        movx    @dptr,a         ; BYTE : No entry currently on table
        inc     dptr        
        clr     C               ; WORD : Current free RAM remain
        mov     a,R2
        subb    a,dpl
        mov     R2,a
        mov     a,R3
        subb    a,dph
        mov     R3,a
        clr     C
        mov     a,R2
        subb    a,#LOW((MaxEntry*3)+4)
        movx    @dptr,a
        inc     dptr
        mov     a,R3
        subb    a,#HIGH((MaxEntry*3)+4)
        movx    @dptr,a
        inc     dptr
        mov     a,dpl           ; WORD : Pointer to next free RAM block
        add     a,#LOW(MaxEntry*3+2)
        movx    @dptr,a
        inc     dptr
        mov     a,dph
        addc    a,#HIGH(MaxEntry*3+2)
        movx    @dptr,a

        ret


 
;****************************************************************************
;
;                     ALLOCATE BLOCK OF MEMORY WITH DMM
;
; Entry:  DPTR is the size of block you whan
; Return: A is the new handle created
;         DPTR is pointer to new block
;
; In case of error, Fatal handler are called. Type of error:
; -Out of memory
; -Out of handle
;
;****************************************************************************
MemALLOC:
         ; R7:R6 is size of block you whan
;

        MOV     R6,dpl          ; Get pointer to Entry table and save size.
        MOV     R7,dph
        MOV     R0,#MemEntry
        MOV     dpl,@R0
        INC     R0
        MOV     dph,@R0


         
        MOVX    a,@dptr         ; Check if too many handle are define
        XRL     a,#MaxEntry
        JNZ     M0
        MOV     a,#0
        JMP     MemERROR

         
M0:     INC     dptr            ; Check out of memory error

        CLR     c               ; Compare free memory with wanted block size
        MOVX    a,@dptr        
        INC     dptr
        SUBB    a,R6
        MOVX    a,@dptr        
        SUBB    a,R7
        JNC     M1
        MOV     a,#1            ; Not enough memory
        JMP     MemERROR

         
M1:     MOV     R0,#MemEntry
        MOV     dpl,@R0
        INC     R0
        MOV     dph,@R0
        MOVX    a,@dptr
        MOV     R2,a                    ; R2 contain number of entry on table
        INC     a
        MOVX    @dptr,a
        INC     dptr

          
        CLR     C               ; substract for wanted block size
        MOVX    a,@dptr
        SUBB    a,R6
        MOVX    @dptr,a
        INC     dptr
        MOVX    a,@dptr
        SUBB    a,R7
        MOVX    @dptr,a
        INC     dptr
         
        MOV     R4,dpl                  ; Save DPTR
        MOV     R5,dph
        MOV     R3,#0                   ; Start with handle #0
M2:     MOV     dpl,R4                  ; set DPTR to begin of entry
        MOV     dph,R5
        MOV     a,R2                    ; R2 is number of entry in table
        JZ      M5                      ; No entry define, create now
        MOV     R0,a                    ; R0 is counter
M3:     MOVX    a,@dptr
        XRL     a,R3
        JNZ     M4
        INC     R3                      ; Same handle finded, try again
        JMP     M2
M4:     INC     dptr
        INC     dptr
        INC     dptr
        DJNZ    R0,M3
M5:                                     ; R3 is handle of new block
         ; DPTR is ptr to end to table, on Free RAM pointer address
;

        MOVX    a,@dptr         ; Get pointer to new block
        MOV     R4,a
        MOV     a,R3
        MOVX    @dptr,a         ; Set new handle at same time
        INC     dptr
        MOVX    a,@dptr
        MOV     R5,a
        MOV     a,R4
        MOVX    @dptr,a         ; Set new block pointer
        INC     dptr
        MOV     a,R5
        MOVX    @dptr,a
        INC     dptr
        MOV     a,R4            ; Set pointer to next free ram block
        ADD     a,R6
        MOVX    @dptr,a
        INC     dptr
        MOV     a,R5
        ADDC    a,R7
        MOVX    @dptr,a
        INC     dptr

        MOV     a,R3            ; Return new handle
        MOV     dpl,R4          ; Return pointer on block
        MOV     dph,R5
        RET

 
;****************************************************************************
;
;                     DEALLOCATE BLOCK OF MEMORY WITH DMM
;
; Entry:  A is the handle to free
; Return: None
;
; In case of error, Fatal handler are called. Type of error:
; -Handle not exist
;
;****************************************************************************
MemFREE:
        MOV     b,a             ; B is handle keeper
         
        MOV     R0,#MemEntry            ; Get entry table pointer
        MOV     a,@R0
        MOV     dpl,a
        INC     R0
        MOV     a,@R0
        MOV     dph,a

        MOVX    a,@dptr                 ; Load number of entry in table
        JNZ     MF0
        MOV     a,#3
        LJMP    MemERROR
MF0:    INC     DPTR
        INC     DPTR
        INC     DPTR
        MOV     R0,a                    ; R0 is entry counter
MF1:    MOVX    a,@dptr
        XRL     a,b
        JZ      MF2
        INC     DPTR
        INC     DPTR
        INC     DPTR
        DJNZ    R0,MF1
        MOV     a,#2
        LJMP    MemERROR                ; handle not finded
MF2:                                    ; handle finded, DPTR is ptr to it                                 
        
        PUSH    dpl
        PUSH    dph
         
; DPTR is ptr to current Entry, R0 is block left before end of table
;
        CJNE    R0,#1,MF3               ; If last block, no garbage collect
        AJMP    MF6

MF3:    MOV     a,R0
        SETB    RS0                     ; Switch to bank #1 for MemCOPY
        MOV     R7,a                    ; R7 is handle scan counter

        INC     dptr                    ; Step handle, go to block
        MOVX    a,@dptr                 ; Get pointer to old block in R3:R2
        MOV     R2,a
        INC     dptr
        MOVX    a,@dptr
        MOV     R3,a
        INC     dptr
        INC     dptr                    ; Step handle of next block
        MOVX    a,@dptr                 ; Get pointer to next block in R1:R0
        MOV     R0,a
        INC     dptr
        MOVX    a,@dptr
        MOV     R1,a
        INC     dptr
        CJNE    R7,#2,MF3a              ; if just one block step handle scan
        SJMP    MF5

MF3a:   DEC     R7
        DEC     R7
MF4:    INC     dptr                    ; Go to Free RAM pointer
        INC     dptr
        INC     dptr
        DJNZ    R7,MF4
           
MF5:    MOVX    a,@dptr                 ; Get pointer to Free RAM in R5:R4
        INC     DPTR
        CLR     C
        SUBB    a,R0
        MOV     R4,a
        MOVX    a,@dptr
        SUBB    a,R1
        MOV     R5,a
        LCALL   MemCOPY                 ; Crush old block with data after
MF6:
        POP     dph
        POP     dpl              ; R0 is handle decounter, dptr ptr to current
         ; DPTR is pointer to entry block, R0 is number of block left.
;

        PUSH    dpl
        PUSH    dph

        DEC     R0                      ; R0 is number of block remain after
        MOV     a,R0
        
        SETB    RS0                     ; Select bank #1 for processing
        MOV     R7,a                    ; R7 is number of block remain after
        INC     dptr                    ; pointer to block size
        MOVX    a,@dptr
        MOV     R4,a                    ; R5:R4 is size of block
        INC     dptr
        MOVX    a,@dptr
        MOV     R5,a
        INC     dptr
        MOV     a,R7
        JZ      MF7                     ; if not last, step handle
        INC     dptr

MF7:    CLR     C
        MOVX    a,@dptr                 ; Find size of deleted block
        SUBB    a,R4
        MOV     R4,a
        INC     dptr
        MOVX    a,@dptr                 ; R5:R4 is size of block
        SUBB    a,R5
        MOV     R5,a

        MOV     a,R7                    ; Modify all block pointer in table
        JZ      MF9
        MOV     R6,a                    ; R6 = counter
        Dec_DPTR                        ; Return to low part of pointer
MF8:    CLR     C
        MOVX    a,@dptr
        SUBB    a,R4
        MOVX    @dptr,a
        INC     dptr
        MOVX    a,@dptr
        SUBB    a,R5
        MOVX    @dptr,a
        INC     dptr
        INC     dptr
        DJNZ    R6,MF8

MF9:    Dec_DPTR                        ; Substract for free ram block pointer
        CLR     C
        MOVX    a,@dptr
        SUBB    a,R4
        MOVX    @dptr,a
        INC     dptr
        MOVX    a,@dptr
        SUBB    a,R5
        MOVX    @dptr,a
        INC     DPTR
        MOV     R6,dpl                  ; R7:R6 pointer at end of table
        MOV     R7,dph

        MOV     R0,#MemEntry            ; Process free space and handle #
        MOV     a,@R0                   ; at begin of table
        MOV     dpl,a
        INC     R0
        MOV     a,@R0
        MOV     dph,a
        MOVX    a,@dptr                 ; minus one entry
        DEC     a
        MOVX    @dptr,a
        INC     dptr
        MOVX    a,@dptr                 ; Add block size to free space
        ADD     a,R4
        MOVX    @dptr,a
        MOVX    a,@dptr
        ADDC    a,R5
        MOVX    @dptr,a

        POP     dph                     ; restore entry pointer
        POP     dpl

        MOV     R2,dpl                  ; Crush entry table
        MOV     R3,dph                  ; R3:R2 destination
        MOV     a,R2
        ADD     a,#3
        MOV     R0,a                    ; R1:R0 source
        MOV     a,R3
        ADDC    a,#0
        MOV     R1,a

        CLR     c                       ; R5:R4 is size of block (minus 3)
        MOV     a,R6
        SUBB    a,dpl
        MOV     R4,a                    ; EndOfTable - CurrentEntryAddress
        MOV     a,R7
        SUBB    a,dph
        MOV     R5,a

        LCALL   MemCOPY



        ret

 
;****************************************************************************
;
;                       GET FREE REMANING MEMORY WITH DMM
;
; Entry:  None
; Return: DPTR is the remaing memory in byte
;
;
;****************************************************************************
MemGETFREE:
        SETB    RS0
        PUSH    ACC

        MOV     R0,#MemEntry            ; DPTR ptr to free memory counter
        MOV     dpl,@R0
        INC     R0
        MOV     dph,@R0
        INC     dptr

        MOVX    a,@dptr
        MOV     R0,a
        INC     dptr
        MOVX    a,@dptr
        MOV     dph,a
        MOV     dpl,R0

        POP     ACC
        CLR     RS0
        RET


 
;****************************************************************************
;
;                   GET BLOCK POINTER FROM HANDLE WITH DMM
;
; Entry:  A is handle
; Return: DPTR is the block pointer
;
;
;****************************************************************************
MemGETPOINTER:
        SETB    RS0
        PUSH    ACC

        MOV     R7,a                    ; Save handle in R7
        MOV     R0,#MemEntry            ; DPTR ptr to entry table
        MOV     dpl,@R0
        INC     R0
        MOV     dph,@R0

        MOVX    a,@dptr                 ; R0 is handle counter
        MOV     R0,a
        INC     dptr
        INC     dptr
        INC     dptr

MGP0:   MOVX    a,@dptr                 ; Scan handle
        XRL     a,R7
        JZ      MGP1
        INC     DPTR        
        INC     DPTR        
        INC     DPTR        
        DJNZ    R0,MGP0
        MOV     a,#2                   ; Handle not finded, error
        LCALL   MemERROR

MGP1:   INC     dptr                    ; Get handle and return it
        MOVX    a,@dptr
        MOV     R0,a
        INC     dptr
        MOVX    a,@dptr
        MOV     dph,a   
        MOV     dpl,R0

        POP     ACC
        CLR     RS0
        RET


 
;****************************************************************************
;
;                    GET POINTER TO FREE MEMORY WITH DMM
;
; Entry:  None
; Return: DPTR is pointer to free memory
;
;
;****************************************************************************
MemGETFREEPTR:
        SETB    RS0
        PUSH    ACC

        MOV     R0,#MemEntry            ; DPTR ptr to entry table
        MOV     dpl,@R0
        INC     R0
        MOV     dph,@R0

        MOVX    a,@dptr                 ; R0 is handle counter
        MOV     R0,a
        INC     dptr
        INC     dptr
        INC     dptr

MGFP0:  INC     dptr                    ; Go to pointer      
        INC     dptr        
        INC     dptr        
        DJNZ    R0,MGFP0

        MOVX    a,@dptr                 ; Get free pointer
        MOV     R0,a
        INC     dptr
        MOVX    a,@dptr                
        MOV     dph,a
        MOV     dpl,R0

        POP     ACC
        CLR     RS0
        RET


 
;****************************************************************************
;
;           COPY EXTERNAL MEMORY BLOCK, CHECK FOR OVERLAPPED BLOCK
;
; Input parameter:
; In register bank #1:
; R1:R0 is source address
; R3:R2 is destination address
; R5:R4 is size of block
;
; Nothing destroyed except parameter in register bank #1
; 
;****************************************************************************
MemCOPY:
        SETB    RS0
        PUSH    dpl
        PUSH    dph
        PUSH    ACC

;************************************************ DETERMINE REVERSE/FORWARD COPY
        CLR     C
        MOV     A,R2
        SUBB    A,R0
        MOV     A,R3
        SUBB    A,R1
        JNC     MemC4

;************************************************ COPY FORWARD DATA
MemC0:  MOV     dpl,R0          ; copy one byte
        MOV     dph,R1
        MOVX    a,@dptr
        MOV     dpl,R2
        MOV     dph,R3
        MOVX    @dptr,a

        INC     R0              ; decrement size, inc source and destination
        INC     R2
        DEC     R4
        CJNE    R0,#0,MemC1
        INC     R1
MemC1:  CJNE    R2,#0,MemC2
        INC     R3
MemC2:  CJNE    R4,#255,MemC3
        DEC     R5

MemC3:  CJNE    R4,#0,MemC0     ; loop 16 bit
        CJNE    R5,#0,MemC0      
      
        SJMP    MemC8  

;************************************************ COPY REVERSE DATA
MemC4:  MOV     a,R0            ; Adjust destination and source
        ADD     a,R4
        MOV     R0,a
        MOV     a,R1
        ADDC    a,R5
        MOV     R1,a
        MOV     a,R2            
        ADD     a,R4
        MOV     R2,a
        MOV     a,R3
        ADDC    a,R5
        MOV     R3,a
        CLR     C
        MOV     a,R0            ; Subtract 1 of destination and source
        SUBB    a,#1
        MOV     R0,a
        MOV     a,R1
        SUBB    a,#0
        MOV     R1,a
        CLR     C
        MOV     a,R2            
        SUBB    a,#1
        MOV     R2,a
        MOV     a,R3
        SUBB    a,#0
        MOV     R3,a

MemC4a: MOV     dpl,R0          ; copy one byte
        MOV     dph,R1
        MOVX    a,@dptr
        MOV     dpl,R2
        MOV     dph,R3
        MOVX    @dptr,a

        DEC     R0              ; decrement size, source and destination
        DEC     R2
        DEC     R4
        CJNE    R0,#255,MemC5
        DEC     R1
MemC5:  CJNE    R2,#255,MemC6
        DEC     R3
MemC6:  CJNE    R4,#255,MemC7
        DEC     R5

MemC7:  CJNE    R4,#0,MemC4a    ; loop 16 bit
        CJNE    R5,#0,MemC4a      
        
MemC8:  POP     ACC
        POP     dph
        POP     dpl
        CLR     RS0
        RET

 ;****************************************************************************
;
;                   HANDLING OF FATAL MEMORY MANAGER ERROR
;
; Error code given in A:
; 0 = Out-of-handle error
; 1 = Out-of-memory error
; 2 = Handle not exist
; 3 = Empty table, cannot free handle
;
;****************************************************************************
MemERROR:
        RET
