;****************************************************************************
;
;                         PACKET RECEIVING VERSION 1.0
;
; BY REMI BILODEAU NOVEMBER 9, 1994
;
; 8051 VERSION
;
;****************************************************************************

 
;****************************************************************************
;
;                  RECEIVE PACKET, CONVERT IT TO BYTE STREAM
;
; Memory requirement:
; Free memory for received packet and packet handle table.
;
; CPU cycle useable for different baud rate:
; 300  baud = 3072 cycles
; 600  baud = 1536 cycles
; 1200 baud = 768  cycles
;
; Function description:
; PacketRXINIT  - Initialize packet receiver
; PacketRECEIVE - Receive packet, put it in RX queue data block.
; WaitTRANSIT   - Wait for transition of packet, time-out are enabled.
;****************************************************************************

 
;****************************************************************************
;
;                            EQUIVALENCE FOR SPM
;
;****************************************************************************
HBaud           EQU     10h     ; Packet baud rate 0 = 300 bauds
                                ;                  1 = 600 bauds
                                ;                  2 = 1200 bauds
                                ;                  3 = 2400 bauds (max)
RXQhandle       EQU     11h     ; Handle of RX queue data block 

RXQsize         EQU     4096    ; Maximum size of RX queue data block



 
;****************************************************************************
;
;            INITIALIZE MEMORY BUFFER FOR RX QUEUE AND OTHER STUFF
;
; Entry:  None
; Return: None
; 
; Variable initialized:
;
;****************************************************************************
PacketRXINIT:
        MOV     dptr,#RXQsize           ; Reserve memory for RX queue block
        CALL    MemALLOC
        MOV     RXQhandle,a
        CLR     a                       ; Clear buffer (size = 0)
        MOVX    @dptr,a
        INC     dptr
        MOVX    @dptr,a

        MOV     a,TMOD                  ; Set TIMER #0 for 16-bit counter
        ANL     a,#0F0h
        ORL     a,#01h
        MOV     TMOD,a             
        SETB    TR0     

        RET

 
;****************************************************************************
;
;                  RECEIVE PACKET AND PUT IT IN BYTE STREAM
;
; Entry:  None
; Return: None
;
; Information:
; Block of data are received and putted in memory block. Assuming DCD are
; detected.
;
; Data are putted in RXQ data block, here is the format of data:
; Word 0 : Size of data (0 = no other block)
; Byte 2 : Data
;  ...
;
; All register are destroyed
;
; R1':R0' = Begin of RX queue (register bank #1)
; R3:R2 = Begin of current writted byte stream
; DPTR = Current write pointer
; R4 = Bit counter for received caracter
; R5 = Current recorded
; R6 = 
; R7 = Counter
;
;****************************************************************************
PacketRECEIVE:
        MOV     a,SlotTime              ; Reset slotcount variable
        MOV     SlotCount,a
         
        MOV     a,RXQhandle     ; Get RX queue start address
        CALL    MemGETPOINTER
        SETB    RS0
        MOV     R0,dpl          ; Set begin of RX queue
        MOV     R1,dph
        CLR     RS0
        MOV     R2,dpl          ; Set Begin of current writted byte stream
        MOV     R3,dph

         
        MOV     a,Tick10ms      ; Set DCD hysteresis
        ADD     a,#2
        MOV     R6,a


RXL0:   CLR     F0              ; Reset completly receiver
              
RXL00:   
        CLR     F1              ; No data are received
        MOV     a,R2            ; Reset receiver, keep sync flag detect status
        MOV     dpl,a
        MOV     a,R3
        MOV     dph,a
        INC     dptr            ; DPTR is write pointer
        INC     dptr
        MOV     R4,#0           ; R4 = Bit in current caracter
        MOV     R5,#0           ; R5 = current recorded caracter

RXL0a:  JB      DCD,RXL1        ; Check DCD with hysteresis
         
        MOV     dpl,R2          ; Clear size header after last packet
        MOV     dph,R3
        CLR     a
        MOVX    @dptr,a
        INC     dptr
        MOVX    @dptr,a

        RET

RXL1:   CALL   WaitTRANSIT      ; Wait transition
        CJNE    a,#7,RXL5       ; Sync flag detected?
         
        JNB     F0,RXL4         ; Jump if sync flag are not detected before
        JNB     F1,RXL4         ; Jump if no data are received

RXL2:   CJNE    R4,#1,RXL00     ; If no integral packet, reset receiver
         
        CLR     c               ; Size of packet in R5:R4
        MOV     a,dpl
        SUBB    a,R2
        MOV     R0,a
        MOV     a,dph
        SUBB    a,R3
        MOV     R1,a
        CLR     C
        MOV     a,R0        
        SUBB    a,#2
        MOV     R0,a
        MOV     a,R1
        SUBB    a,#0
        MOV     R1,a
        
         
        MOV     a,dpl           ; exchange R3:R2 <-> DPTR
        MOV     dpl,R2
        MOV     R2,a
        MOV     a,dph
        MOV     dph,R3
        MOV     R3,a

         
        MOV     a,R0
        MOVX    @dptr,a
        INC     dptr
        MOV     a,R1
        MOVX    @dptr,a

        JMP     RXL0            ; start new packet receive  

RXL4:   SETB    F0              ; flag finded
        MOV     R4,#0           ; R4 = Bit in current caracter, reset value
        MOV     R5,#0           ; R5 = current recorded caracter, reset value
        JMP     RXL10


RXL5:   JNZ     RXL6            ; Valid transition detected, store it
        JMP     RXL0            ; Invalid bit count, receiver total reset

RXL6:   MOV     b,a             ; Save transition for bit stuffering test
         
        DEC     a               ; Setup bit number
        JZ      RXL8a           ; transition = 1, no '1' bit here
        MOV     R7,a            ; R7 is bit counter to store '1' bit
        MOV     a,R5            

RXL7:   RR      a               ; A contain recorded caracter
        SETB    Abit7
        INC     R4              ; increment bit counter

        CJNE    R4,#8,RXL8      ; store byte if need
        JNB     F0,RXL7a        ; Do not store byte if sync flag not detected
        MOVX    @dptr,a
        INC     dptr
        SETB    F1              ; Data are writted
RXL7a:  MOV     R4,#0           ; R4 = Bit in current caracter, reset value
        CLR     a               ; R5 = current recorded caracter, reset value

RXL8:   DJNZ    R7,RXL7
        MOV     R5,a            ; Store in R5 the caracter


RXL8a:  MOV     a,b
        CJNE    a,#6,RXL9       ; if no stuffered bit, store the '0' bit
        JMP     RXL10

RXL9:    
        MOV     a,R5        
        RR      a               ; Store '0' in caracter
        CLR     Abit7
        MOV     R5,a
        INC     R4              ; increment bit counter

        CJNE    R4,#8,RXL10     ; store byte if need
        JNB     F0,RXL6a
        MOVX    @dptr,a
        INC     dptr
        SETB    F1              ; Data are writted
 RXL6a: MOV     R4,#0           ; R4 = Bit in current caracter, reset value
        MOV     R5,#0           ; R5 = current recorded caracter, reset value


RXL10:   
        SETB    RS0
        CLR     C
        MOV     a,dpl           ; Calculate size of byte received
        SUBB    a,R0
        MOV     R2,a
        MOV     a,dph
        SUBB    a,R1
        MOV     R3,a
        CLR     C
        MOV     a,R2
        SUBB    a,#low(RXQsize-16)
        MOV     a,R3
        SUBB    a,#high(RXQsize-16)
        JC      RXL11
        CLR     RS0
        JMP     RXL0

RXL11:  CLR     RS0

        JMP     RXL0a           ; size is ok, continue receiving

 
;****************************************************************************
;
;                  WAIT TRANSITION AND RETURN THE BIT LENGTH
;
; Entry:  None
; Return: Return bit length of transition in A, between last transition, valid
;         range is 0 to 7, 0 indicate invalid transition time.
; 
; Variable used:
; HBaud  -  Packet baud rate
;
; Information:
; baud rate accepted is 300,600 and 1200 bauds (1200 bauds is locked now)
;
; Register A,B destroyed
; 
;****************************************************************************
WaitTRANSIT:
         
        JNB     PktIN,Wait1

Wait0:  MOV     a,TH0                   ; Wait for zero level
        ANL     a,#11100000b
        JNZ     OverT
        JB      PktIN,Wait0
        JMP     FindT
        
Wait1:  MOV     a,TH0                   ; Wait for one level
        ANL     a,#11100000b
        JNZ     OverT
        JNB     PktIN,Wait1

                                
FindT:  MOV     a,TL0           ; add 1/2 bit to time
        ADD     a,#80h
        MOV     a,TH0
        ADDC    a,#01h

        MOV     TL0,#0          ; Reset timer
        MOV     TH0,#0

        MOV     b,#3            ; time/768 to find bit time
        DIV     ab

        MOV     b,a             ; Test bit overflow
        ANL     a,#11111000b
        JNZ     OverT
        MOV     a,b
        RET

OverT:   
;---------------------------- BAD TRANSITION DETECTED ON PACKET PORT
        MOV     TL0,#0
        MOV     TH0,#0
        CLR     a                       ; Bad transition length

        RET
