;****************************************************************************
;
;                      SERIAL PORT MANAGER VERSION 1.0
;
; BY REMI BILODEAU OCTOBER 21, 1994
;
; 8051 VERSION
;
;****************************************************************************
 
;****************************************************************************
;
;                      SERIAL PORT MANAGER DESCRIPTION
;
; Memory requirement:
; RECEIVE BUFFER  : from 16 to 256 bytes with DMM, Fixed to 32 bytes V1.0
; TRANSMIT BUFFER : from 16 to 256 bytes with DMM, Fixed to 128 bytes V1.0 
;
; Function description:
; SerialINIT      : Initialize serial port, Timer 1 and reserve memory 
; SerialRX        : Wait byte for serial port and return it in A
; SerialGET       : Check for byte in serial, return it in A else carry are set
; SerialTX        : Send data to serial port, data in A
; SerialHEX       : Send in hex format a byte in A
; SerialNUMBER    : Send in decimal format a byte in A
; SerialSTRING    : Send ASCII string to serial port with CRLF process
;
;****************************************************************************


 
;****************************************************************************
;
;                            EQUIVALENCE FOR SPM
;
;****************************************************************************
SerRXptr        EQU     32h             ; Serial receive buffer pointer
SerRXreadCnt    EQU     34h             ; Serial receive buffer read address
SerRXwriteCnt   EQU     35h             ; Serial receive buffer write address
SerTXptr        EQU     36h             ; Serial transmit buffer pointer
SerTXreadCnt    EQU     38h             ; Serial transmit buffer read address
SerTXwriteCnt   EQU     39h             ; Serial transmit buffer write address
SerTVALUE       EQU     3Ah             ; Serial timer value saving 

SerRXmax        EQU     32              ; Maximum byte in receive queue
SerTXmax        EQU     128             ; Maximum byte in transmit queue

TXemptyReady    EQU     0               ; FLAG: Tx buffer are empty are ready!
CRLFflag        EQU     1               ; FLAG: CRLF are send else CR only
SerialECHO      EQU     2               ; FLAG: If set all RX char are echoed

 
;****************************************************************************
;
;                      SERIAL PORT INTERRUPT HANDLER
;
; Receive or transmit bufferred data
;
;****************************************************************************
SerialINT:
        PUSH    dph
        PUSH    dpl
        PUSH    acc
        PUSH    psw

        JB      ti,TXflag       ; receiver contains character ?
        JNB     ri,fin
         
         

        MOV     a,SerRXreadCnt          ; Check if buffer are full
        JNZ     Rf0
        ADD     a,#SerRXmax
Rf0:    DEC     a
        MOV     dpl,a
        XRL     a,SerRXwriteCnt
        JZ      FinRx
        MOV     a,dpl

        JNZ     Rf1                     ; Check to set CTS
        ADD     a,#SerRXmax    
Rf1:    DEC     a                       
        XRL     a,SerRXwriteCnt
        JNZ     NoCTS
        SETB    CTS

NoCTS:  MOV     dpl,SerRXptr            ; Load buffer pointer
        MOV     dph,SerRXptr+1

        MOV     a,dpl                   ; Set DPTR to write position
        ADD     a,SerRXwriteCnt
        MOV     dpl,a
        JNC     rf2
        INC     dph

rf2:    INC     SerRXwriteCnt           ; Update write position
        MOV     a,SerRXwriteCnt
        CLR     C
        SUBB    a,#SerRXmax
        JC      NoOV
        MOV     SerRXwriteCnt,a

NoOV:   MOV     a,SBUF                  ; Write byte to buffer
        MOVX    @dptr,a


FinRx:  CLR     RI                      ; Clear receiver flag
        JNB     ti,fin

TXflag:   
        MOV     a,SerTXwriteCnt ; Check if char avail. else TXempty and Ready
        XRL     a,SerTXreadCnt
        JNZ     Stf0
        SETB    TXemptyReady
        JMP     FinTX

Stf0:   MOV     dpl,SerTXptr    ; Set DPTR to serial transmit buffer
        MOV     dph,SerTXptr+1

        MOV     a,dpl           ; Set DPTR to read position
        ADD     a,SerTXreadCnt
        MOV     dpl,a
        JNC     Stf1
        INC     dph

Stf1:   MOVX    a,@dptr         ; Read byte and send it to serial port
        MOV     SBUF,a

        INC     SerTXreadCnt    ; Update read position
        MOV     a,SerTXreadCnt
        CLR     C
        SUBB    a,#SerTXmax
        JC      FinTX
        MOV     SerTXreadCnt,a

FinTX:  CLR     TI              ; Clear transmission flag
 


fin:    POP     psw
        POP     acc
        POP     dpl
        POP     dph
        RETI


 
;****************************************************************************
;
;                      INITIALIZE SERIAL PORT MANAGER
;
; -Configure 8051 hardware for serial port use
; -Reserve memory for buffer, 256 byte each
; -Reset all variable
;
;****************************************************************************
SerialINIT:
         
        MOV     dptr,#SerRXmax  ; Allocate memory for receive buffer
        LCALL   MemALLOC
        MOV     SerRXptr,dpl
        MOV     SerRXptr+1,dph
        MOV     SerRXreadCnt,#0
        MOV     SerRXwriteCnt,#0

        MOV     dptr,#SerTXmax  ; Allocate memory for transmit buffer
        LCALL   MemALLOC
        MOV     SerTXptr,dpl
        MOV     SerTXptr+1,dph
        MOV     SerTXreadCnt,#0
        MOV     SerTXwriteCnt,#0

        SETB    TXemptyReady    ; TX buffer are empty, Serial port are ready!
         
        mov     SCON,#52h               ; Serial init for 9600 1n8
        mov     a,TMOD
        anl     a,#0Fh
        orl     a,#20h
        mov     TMOD,a                  ; Set timer 1 mode
        mov     TH1,#0FDh               ; Set timer reload value
        MOV     SerTVALUE,#0FDh         ; Save value of timer for serial port
        setb    TR1                     ; Start timer
        setb    ES                      ; Enable serial interrupt
        RET

 
;****************************************************************************
;
;                       RECEIVE BYTE FROM SERIAL PORT
;
; Entry:  None
; Return: A is the caracter received
;
;****************************************************************************
SerialRX:
        PUSH    dpl
        PUSH    dph
        SETB    RS0             ; Select bank #1 for push general register

Srx0:   MOV     R0,#SerRXwriteCnt ; Wait a for caracter in buffer
        MOV     a,@R0
        DEC     R0              
        XRL     a,@R0
        JZ      Srx0

        CLR     ES              ; Clear serial interrupt 

        MOV     R0,#SerRXptr    ; Set DPTR to serial receive buffer
        MOV     dpl,@R0
        INC     R0
        MOV     dph,@R0

        MOV     R0,#SerRXreadCnt
        MOV     a,dpl           ; Set DPTR to read position
        ADD     a,@R0
        MOV     dpl,a
        MOV     a,dph
        ADDC    a,#0
        MOV     dph,a 

        MOVX    a,@dptr         ; Read received byte
        PUSH    ACC

        INC     @R0             ; Update read position
        MOV     a,@R0
        CLR     c
        SUBB    a,#SerRXmax
        JC      Srx1
        MOV     @R0,a

Srx1:   POP     ACC             ; Restore getted byte

        SETB    ES              ; Reenable serial interrupt
        CLR     CTS             ; At least one byte are received

        CLR     RS0
        POP     dph
        POP     dpl
        ret

 
;****************************************************************************
;
;                     CHECK AND RECEIVE BYTE FROM SERIAL PORT
;
; Entry:  None
; Return: A is the caracter received, else carry are set
;
;****************************************************************************
SerialGET:
        PUSH    dpl
        PUSH    dph
        SETB    RS0             ; Select bank #1 for push general register

        MOV     R0,#SerRXwriteCnt ; Wait a for caracter in buffer
        MOV     a,@R0
        DEC     R0              
        XRL     a,@R0
        JNZ     Sg0

        SETB    c               ; No byte availaible
        CLR     RS0
        POP     dph
        POP     dpl
        RET

Sg0:    CLR     ES              ; Clear serial interrupt 

        MOV     R0,#SerRXptr    ; Set DPTR to serial receive buffer
        MOV     dpl,@R0
        INC     R0
        MOV     dph,@R0

        MOV     R0,#SerRXreadCnt
        MOV     a,dpl           ; Set DPTR to read position
        ADD     a,@R0
        MOV     dpl,a
        MOV     a,dph
        ADDC    a,#0
        MOV     dph,a 

        MOVX    a,@dptr         ; Read received byte
        PUSH    ACC

        INC     @R0             ; Update read position
        MOV     a,@R0
        CLR     c
        SUBB    a,#SerRXmax
        JC      Sg1
        MOV     @R0,a

Sg1:    POP     ACC             ; Restore getted byte

        SETB    ES              ; Reenable serial interrupt
        CLR     CTS             ; At least one byte are received

        CLR     c
        CLR     RS0
        POP     dph
        POP     dpl
        RET


 
;****************************************************************************
;
;                         TRANSMIT BYTE TO SERIAL PORT
;
; Entry:  A is the caracter to transmit
; Return: None
;
;****************************************************************************
SerialTX:
        SETB    RS0                     ; Set alternative register set
        PUSH    dpl
        PUSH    dph
        PUSH    ACC          

        MOV     dpl,SerTXptr            ; Set DPTR to serial transmit buffer
        MOV     dph,SerTXptr+1

Stx0:   MOV     a,SerTXreadCnt          ; Wait for place in buffer
        JNZ     Stx1                    
        ADD     a,#SerTXmax
Stx1:   DEC     a
        XRL     a,SerTXwriteCnt
        JZ      Stx0

        CLR     ES                      ; Clear serial interrupt

        MOV     a,dpl                   ; Set DPTR to write position
        ADD     a,SerTXwriteCnt
        MOV     dpl,a
        MOV     a,dph
        ADDC    a,#0
        MOV     dph,a        

        INC     SerTXwriteCnt           ; Update write position
        MOV     a,SerTXwriteCnt
        CLR     c
        SUBB    a,#SerTXmax
        JC      NoOV2
        MOV     SerTXwriteCnt,a

NoOV2:  POP     ACC
        PUSH    ACC                     ; Write byte in buffer
        MOVX    @dptr,a                 

        JNB     TXemptyReady,Stx2       ; Trig transmission if needed
        SETB    TI
        CLR     TXemptyReady

Stx2:   SETB    ES                      ; Re-enable serial interrupt

        POP     ACC                     ; Pop register
        POP     dph                     
        POP     dpl
        CLR     RS0                     ; Reset good set of register

        RET


 
;****************************************************************************
;
;                SEND TO SERIAL PORT A ASCII HEX 8-BIT NUMBER
;
; Entry:  A is the number to convert
; Return: None
;
;****************************************************************************
SerialHex:
        PUSH    dpl
        PUSH    dph
        PUSH    ACC
        PUSH    b
        
        MOV     b,a
        SWAP    a
        ANL     a,#00001111b
        MOV     dptr,#Hexa   
        MOVC    a,@a+dptr
        CALL    SerialTX

        MOV     a,b
        ANL     a,#00001111b
        MOV     dptr,#Hexa   
        MOVC    a,@a+dptr
        CALL    SerialTX

        POP     b
        POP     ACC
        POP     dph
        POP     dpl        
        RET

Hexa:   db '0123456789ABCDEF'


 
;****************************************************************************
;
;                    SEND NUMBER IN ASCII TO SERIAL PORT
; 
; Entry:  A is the binary number to convert
; Return: None
;
;****************************************************************************
SerialNUMBER:
        PUSH    ACC
        PUSH    b
        SETB    RS0

        MOV     R0,#1           ; Convert number in R1-R2-R3
Sendn1: MOV     b,#10
        DIV     ab
        PUSH    ACC
        MOV     a,b
        ADD     a,#48
        MOV     @R0,a
        INC     R0
        POP     ACC
        JNZ     Sendn1
        DEC     R0

        MOV     a,@R0           ; print 1, 2 or 3 digit
        MOV     b,R0
        CALL    SerialTX
        MOV     R0,b
        MOV     a,R0
        DEC     a
        JZ      Sendn2
        MOV     R0,a   
        MOV     a,@R0
        MOV     b,R0
        CALL    SerialTX
        MOV     R0,b
        MOV     a,R0
        DEC     a
        JZ      Sendn2
        MOV     R0,a   
        MOV     a,@R0
        CALL    SerialTX

Sendn2: CLR     RS0
        POP     b
        POP     ACC
        RET


  
;****************************************************************************
;
;                      SERIAL PORT STRING TRANSMISION
;
; Entry:  DPTR pointer to rom ASCIIZ string
; Return: none
;
;****************************************************************************
SerialSTRING:
       PUSH     ACC
Sstr0: CLR      a
       MOVC     a,@a+dptr               ; Get byte
       CJNE     a,#00h,more             ; if 0 then done
       POP      ACC
       RET
more:  CJNE     a,#13,more2             ; process CRLF/CR state
       JNB      CRLFflag,more2
       CALL     SerialTX
       MOV      a,#10  
more2: CALL     SerialTX
       INC      dptr
       JMP      Sstr0



